#!/usr/bin/perl -w -I.
use strict;

#  ----------------------------------------------------------------------
# | Open Archives Initiative Harvesting Framework version 1.1            |
# | Hussein Suleman                                                      |
# | September 2001                                                       |
#  ----------------------------------------------------------------------
# |  Virginia Polytechnic Institute and State University                 |
# |  Department of Computer Science                                      |
# |  Digital Library Research Laboratory                                 |
#  ----------------------------------------------------------------------

# Archive-specific routines

package NDLTDArchive;

use DBI;
use Error;

sub new
{
   my $classname = shift;
   
   # read in configuration if it exists
   my $self;
   if (-e 'cfg.pl')
   {
      $self = do 'cfg.pl';
   }
   
   # hard-code defaults for missing configuration values
   my %harddefaults =
   ( 
      submissionPolicy     => 'Submissions are only made internally by students',
      recordlimit          => '100',
      publisherauthority   => 'University Libraries',
      language             => 'en',
      database             => 'DBI:mysql:etd_available:localhost',
      archiveId            => 'ArchiveID',
      dbpassword           => '',
      adminEmail           => 'root@localhost',
      publisher            => 'Publisher',
      dbusername           => 'mysqldump',
      publisherinitials    => 'XXX',
      publisherlocation    => 'LLL',
      resumptionseparator  => '!',
      content              => 'Theses and Dissertations produced by students',
      sampleIdentifier     => 'etd-1234567890',
      repositoryName       => 'XXX Electronic Thesis and Dissertation Archive',
      metadataPolicy       => 'Metadata may be used by commercial and non-commercial users',
      dataPolicy           => 'Full texts are individually tagged and the rights statements must be adhered to'
   );
   foreach my $key (keys %harddefaults)
   {
      if (! defined $self->{$key})
      { 
         $self->{$key} = $harddefaults{$key}; 
      }
   }

   bless $self, $classname;
   return $self;
}

sub Archive_Identify
{
   my $self = shift;

   {
      repositoryName => $self->{'repositoryName'},
      adminEmail     => 'mailto:'.$self->{'adminEmail'},
      description    => [
         {
            'oai-identifier' => [[ 
               { 
                  xmlns => 'http://www.openarchives.org/OAI/1.1/oai-identifier',
                  'xsi:schemaLocation' => 'http://www.openarchives.org/OAI/1.1/oai-identifier http://www.openarchives.org/OAI/1.1/oai-identifier.xsd'
               }, 
               {
                  scheme               => 'oai',
                  repositoryIdentifier => $self->{'archiveId'},
                  delimiter            => ':',
                  sampleIdentifier     => "oai:$self->{'archiveId'}:$self->{'sampleIdentifier'}",
                  mdorder              => [ qw ( scheme repositoryIdentifier delimiter sampleIdentifier ) ]
               } 
            ]]
         },
         {
            eprints => [[
               {
                  xmlns => 'http://www.openarchives.org/OAI/1.1/eprints',
                  'xsi:schemaLocation' => 'http://www.openarchives.org/OAI/1.1/eprints http://www.openarchives.org/OAI/1.1/eprints.xsd'                
               },
               {
                  content          => { text => $self->{'content'} },
                  metadataPolicy   => { text => $self->{'metadataPolicy'} },
                  dataPolicy       => { text => $self->{'dataPolicy'} },
                  submissionPolicy => { text => $self->{'submissionPolicy'} },
                  mdorder          => [ qw ( content metadataPolicy dataPolicy submissionPolicy ) ]
               }
            ]]
         }
      ]
   };
}

sub Archive_ListSets
{
   [[ 'All', 'All theses and dissertations' ]]
}

sub BuildDBRecord
{
   my ($self, $dbh, $identifier, $rowref) = @_;

   my $sth2 = $dbh->prepare ("select keyword from keyword_by_urn where urn=\'$identifier\'");
   $sth2->execute;
   my $sth3 = $dbh->prepare ("select filename from filename_by_urn where urn=\'$identifier\'");
   $sth3->execute;
   my $sth4 = $dbh->prepare ("select advisor_name, advisor_title from advisor_by_urn where urn=\'$identifier\'");
   $sth4->execute;
 
   my ($akeyword, $acommmember, $acommtitle, $afile);

   $rowref->{'keywords'} = [];
   while ($akeyword = ($sth2->fetchrow_array)[0])
   { push (@{$rowref->{'keywords'}}, $akeyword); }

   $rowref->{'files'} = [];
   while ($afile = ($sth3->fetchrow_array)[0])
   { push (@{$rowref->{'files'}}, $afile); }

   $rowref->{'committee'} = [];
   while (($acommmember, $acommtitle) = $sth4->fetchrow_array)
   { push (@{$rowref->{'committee'}}, [$acommmember, $acommtitle]); }

   $rowref->{'archiveId'} = $self->{'archiveId'};
   $rowref->{'publisher'} = $self->{'publisher'};
   $rowref->{'publisherinitials'} = $self->{'publisherinitials'};
   $rowref->{'publisherauthority'} = $self->{'publisherauthority'};
   $rowref->{'publisherlocation'} = $self->{'publisherlocation'};
   $rowref->{'language'} = $self->{'language'};

   $sth2->finish;
   $sth3->finish;
   $sth4->finish;
}

sub Archive_GetRecord
{
   my ($self, $identifier) = @_;

   my @fulldata = split (':', $identifier);
   if (! defined $fulldata[2])
   {
      $fulldata[2] = '';
   }

   my $dbh = DBI->connect($self->{'database'}, $self->{'dbusername'}, $self->{'dbpassword'}, { PrintError=>0 });
   if (!$dbh)
   {
      new Error ('Cannot connect to database - run configure.pl again', 500);
      exit;
   }
   
   my $sth = $dbh->prepare ("select * from etd_main where urn=\'$fulldata[2]\'");
   $sth->execute;

   my $rowref;
   if ($rowref = $sth->fetchrow_hashref)
   {
      $self->BuildDBRecord ($dbh, $fulldata[2], $rowref);
   }

   $sth->finish;
   $dbh->disconnect;

   $rowref;
}

sub Archive_ListRecords
{
   my ($self, $set, $from, $until, $metadataPrefix, $resumptionToken) = @_;

   my ($offset);
   if ($resumptionToken eq '')
   {
      $offset = 0;
   }
   else
   {
      my @rdata = split ($self->{'resumptionseparator'}, $resumptionToken);
      ($set, $from, $until, $metadataPrefix, $offset) = @rdata;
   }

   my $dbh = DBI->connect($self->{'database'}, $self->{'dbusername'}, $self->{'dbpassword'}, { PrintError=>0 });
   if (!$dbh)
   {
      new Error ('Cannot connect to database - run configure.pl again', 500);
      exit;
   }
   my $sth;

   # construct query
   my $sqlquery = "select";
   if ($metadataPrefix ne '') 
   { $sqlquery .= " * from etd_main"; }
   else
   { $sqlquery .= " urn from etd_main"; }

   if ($from eq '')
   {
      if ($until eq '') 
      { }
      else
      { $sqlquery = $sqlquery." where (adate <= \'$until\')"; }
   }
   else
   { 
      if ($until eq '')
      { $sqlquery = $sqlquery." where (adate >= \'$from\')"; }
      else
      { $sqlquery = $sqlquery." where (adate >= \'$from\') and (adate <= \'$until\')"; }
   }

   my $limit = $self->{'recordlimit'} + 1;
   $sqlquery = $sqlquery." limit $offset,$limit";
   $sth = $dbh->prepare ($sqlquery);
   $sth->execute;

   my $rowref;
   my @allrows = ();
   my $gotmore = 0;
   my $count = 0; 

   while ($rowref = $sth->fetchrow_hashref)
   {
      if ($count == $self->{'recordlimit'})
      { $gotmore = 1; }
      else
      {
         $count++;
         if ($metadataPrefix eq '')
         { $rowref->{'urn'} = "oai:$self->{'archiveId'}:$rowref->{'urn'}"; }
         else
         { $self->BuildDBRecord ($dbh, $rowref->{'urn'}, $rowref); }
         push @allrows, { %$rowref };
      }
   }

   $sth->finish;
   $dbh->disconnect;

   $resumptionToken = '';
   if ($gotmore == 1)
   {
      $resumptionToken = join ($self->{'resumptionseparator'}, ($set,$from,$until,$metadataPrefix,$offset+$self->{'recordlimit'}));
   }

   ( \@allrows, $resumptionToken, $metadataPrefix );
}

sub Archive_ListIdentifiers
{
   my ($self, $set, $from, $until, $resumptionToken) = @_;

   $self->Archive_ListRecords ($set, $from, $until, '', $resumptionToken);
}

1;
