#!/usr/bin/perl -w -I.
use strict;

#  ----------------------------------------------------------------------
# | Open Archives Initiative Harvesting Framework version 1.1            |
# | Hussein Suleman                                                      |
# | July 2001                                                            |
#  ----------------------------------------------------------------------
# |  Virginia Polytechnic Institute and State University                 |
# |  Department of Computer Science                                      |
# |  Digital Library Research Laboratory                                 |
#  ----------------------------------------------------------------------

# Installation :
#   copy all file into a directory from which you can execute scripts
#
# Testing :
#   use the repository explorer (http://purl.org/net/oai_explorer)
#   to test your interface

package OAI;

use Error;
use Parameter;
use Utility;

sub new
{
   my ($classname, $archive, $metadata) = @_;

   my $self = {
      xmlnsprefix     => 'http://www.openarchives.org/OAI/1.1',
      protocolversion => '1.1',
      archive         => $archive,
      metadata        => $metadata,
      utility         => new Utility,
      cgi             => new Parameter
   };

   bless $self, $classname;

   $self->{'archive'}->{'utility'} = $self->{'utility'};
   $self->{'metadata'}->{'utility'} = $self->{'utility'};

   # catch uninititialized errors
   if (! defined $self->{'cgi'}->{'verb'})
   { $self->{'cgi'}->{'verb'} = ''; }

   # run appropriate handler procedure
   if ($self->{'cgi'}->{'verb'} eq 'Identify')
   { $self->Identify; }
   elsif ($self->{'cgi'}->{'verb'} eq 'ListMetadataFormats')
   { $self->ListMetadataFormats; }
   elsif ($self->{'cgi'}->{'verb'} eq 'GetRecord')
   { $self->GetRecord; }
   elsif ($self->{'cgi'}->{'verb'} eq 'ListSets')
   { $self->ListSets; }
   elsif ($self->{'cgi'}->{'verb'} eq 'ListRecords')
   { $self->ListRecords; }
   elsif ($self->{'cgi'}->{'verb'} eq 'ListIdentifiers')
   { $self->ListIdentifiers; }
   else
   { new Error ('unsupported verb', 400); }

   return $self;
}

# output XML HTTP header

sub xmlheader
{
   my ($self) = @_;

   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime (time);
   my $timezone = sprintf ("%+03d:00", ((((($hour-(gmtime)[2]) % 24) + 11) % 24) - 11));
   my $datestring = sprintf ("%04d-%02d-%02dT%02d:%02d:%02d%s",
                    $year+1900, $mon+1, $mday, $hour, $min, $sec,
                    $timezone);

   print "Content-type: text/xml\n";
   print "\n";
   print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n";
   print "<$self->{'cgi'}->{'verb'} xmlns=\"$self->{'xmlnsprefix'}/OAI_$self->{'cgi'}->{'verb'}\" ";
   print "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ";
   print "xsi:schemaLocation=\"$self->{'xmlnsprefix'}/OAI_$self->{'cgi'}->{'verb'} ";
   print "$self->{'xmlnsprefix'}/OAI_$self->{'cgi'}->{'verb'}.xsd\">\n\n";
   print "<responseDate>$datestring</responseDate>\n";
   print "<requestURL>";
   print $self->{'utility'}->lclean ($self->{'cgi'}->{'requestURL'});
   print "</requestURL>\n\n";
}

# output XML HTTP footer

sub xmlfooter
{
   my ($self) = @_;

   print "\n</$self->{'cgi'}->{'verb'}>\n";
}

# verbs

sub Identify
{
   my ($self) = @_;

   my $identity = $self->{'archive'}->Archive_Identify;
   $identity->{'protocolVersion'} = $self->{'protocolversion'};
   $identity->{'baseURL'} = $self->{'cgi'}->{'baseURL'};
   $identity->{'mdorder'} = [ qw ( repositoryName baseURL protocolVersion adminEmail description ) ];

   $self->xmlheader;
   print $self->{'utility'}->FormatXML ($identity);
   $self->xmlfooter;
}

sub ListMetadataFormats
{
   my ($self) = @_;

   $self->xmlheader;
   my $i;
   foreach $i (keys %{$self->{'metadata'}->{'metadatanamespace'}})
   {
      print "<metadataFormat>\n";
      print "<metadataPrefix>$i</metadataPrefix>\n";
      print "<schema>$self->{'metadata'}->{'metadataschema'}->{$i}</schema>\n";
      print "<metadataNamespace>$self->{'metadata'}->{'metadatanamespace'}->{$i}</metadataNamespace>\n";
      print "</metadataFormat>\n";
   }
   $self->xmlfooter;
}

sub ListSets
{
   my ($self) = @_;

   $self->xmlheader;
   
   my $setlist = $self->{'archive'}->Archive_ListSets;
   my $item;
   
   foreach $item (@$setlist)
   {
      print "<set>\n";
      print "  <setSpec>".$self->{'utility'}->lclean ($$item[0])."</setSpec>\n";
      print "  <setName>".$self->{'utility'}->lclean ($$item[1])."</setName>\n";
      print "</set>\n";
   }
   
   $self->xmlfooter;
}

sub GetRecord
{
   my $self = shift;

   my ($identifier) = defined $self->{'cgi'}->{'identifier'} ? $self->{'cgi'}->{'identifier'} : '';
   my ($metadataPrefix) = defined $self->{'cgi'}->{'metadataPrefix'} ? $self->{'cgi'}->{'metadataPrefix'} : '';

   if ($metadataPrefix eq '')
   {
      new Error ("missing metadataPrefix parameter", 400);
      return;
   }
   if ($identifier eq '')
   {
      new Error ("missing identifier parameter", 400);
      return;
   }

   my $rowref = $self->{'archive'}->Archive_GetRecord ($identifier);

   $self->xmlheader;
   if ($rowref)
   {
      print $self->{'metadata'}->Format ($rowref, $metadataPrefix);
   }
   $self->xmlfooter;
}

sub ListRecords
{
   my $self = shift;

   my ($set, $from, $until, $metadataPrefix);
   my ($resumptionToken);

   $resumptionToken = defined $self->{'cgi'}->{'resumptionToken'} ? $self->{'cgi'}->{'resumptionToken'} : '';
   if ($resumptionToken eq '')
   {
      $set = defined $self->{'cgi'}->{'set'} ? $self->{'cgi'}->{'set'} : '';
      $from = defined $self->{'cgi'}->{'from'} ? $self->{'cgi'}->{'from'} : '';
      $until = defined $self->{'cgi'}->{'until'} ? $self->{'cgi'}->{'until'} : '';
      $metadataPrefix = defined $self->{'cgi'}->{'metadataPrefix'} ? $self->{'cgi'}->{'metadataPrefix'} : '';

      if ($metadataPrefix eq '')
      {
         new Error ("missing metadataPrefix parameter", 400);
         return;
      }
      if (($from ne '') && (!($self->{'utility'}->DateisValid ($from))))
      {
         new Error ("illegal from parameter", 400);
         return;
      }
      if (($until ne '') && (!($self->{'utility'}->DateisValid ($until))))
      {
         new Error ("illegal until parameter", 400);
         return;
      }
   }

   my ($allrows, $rowref);
   ($allrows, $resumptionToken, $metadataPrefix) =  $self->{'archive'}->Archive_ListRecords ($set, $from, $until, $metadataPrefix, $resumptionToken);

   $self->xmlheader;
   foreach $rowref (@$allrows)
   { 
      print $self->{'metadata'}->Format ($rowref, $metadataPrefix);
   }
   if ($resumptionToken ne '')
   {
      print "<resumptionToken>$resumptionToken</resumptionToken>\n";
   }
   $self->xmlfooter;
}

sub ListIdentifiers
{
   my $self = shift;

   my ($set, $from, $until);
   my ($resumptionToken);

   $resumptionToken = defined $self->{'cgi'}->{'resumptionToken'} ? $self->{'cgi'}->{'resumptionToken'} : '';
   if ($resumptionToken eq '')
   {
      $set = defined $self->{'cgi'}->{'set'} ? $self->{'cgi'}->{'set'} : '';
      $from = defined $self->{'cgi'}->{'from'} ? $self->{'cgi'}->{'from'} : '';
      $until = defined $self->{'cgi'}->{'until'} ? $self->{'cgi'}->{'until'} : '';

      if (($from ne '') && (!($self->{'utility'}->DateisValid ($from))))
      {
         new Error ("illegal from parameter", 400);
         return;
      }
      if (($until ne '') && (!($self->{'utility'}->DateisValid ($until))))
      {
         new Error ("illegal until parameter", 400);
         return;
      }
   }

   my ($allrows, $rowref);
   ($allrows, $resumptionToken) = $self->{'archive'}->Archive_ListIdentifiers ($set, $from, $until, $resumptionToken);

   $self->xmlheader;
   foreach $rowref (@$allrows)
   {
      print "<identifier>$rowref->{'urn'}</identifier>"
   }
   if ($resumptionToken ne '')
   {
      print "<resumptionToken>$resumptionToken</resumptionToken>\n";
   }
   $self->xmlfooter;
}

1;


