#!/usr/bin/perl -I. -w
use strict;

#  ----------------------------------------------------------------------
# | NDLTD OAI Data Provider Configurator version 1.0                     |
# | Hussein Suleman                                                      |
# | September 2001                                                       |
#  ----------------------------------------------------------------------
# |  Virginia Polytechnic Institute and State University                 |
# |  Department of Computer Science                                      |
# |  Digital Library Research Laboratory                                 |
#  ----------------------------------------------------------------------

# configure the NDLTD OAI data provider layer


# read in configuration from cfg.pl and set best-guess defaults for 
# missing values
sub ReadConfig
{
   my $config;
   
   if (-e 'cfg.pl')
   { $config = do 'cfg.pl'; }
   else
   { $config = {}; }
   
   if ((! defined $config->{archiveId}) || ($config->{archiveId} eq ''))
   {
      $config->{archiveId} = '';
      
      # guess archive id from hostname (assuming position of institution name)
      my $hostname = `hostname`; 
      if ((defined $hostname) && ($hostname ne ''))
      {
         $hostname =~ s/[\r\n\t\s]//go;
         
         my @hostparts = split (/[\.]/, $hostname);
         if ($#hostparts >= 2)
         {
            if (uc ($hostparts[$#hostparts-1]) eq 'ac')
            { $config->{archiveId} = uc ($hostparts[$#hostparts-2]).'ETD'; }
            else
            { $config->{archiveId} = uc ($hostparts[$#hostparts-1]).'ETD'; }
         }
      }
   }

   if ((! defined $config->{adminEmail}) || ($config->{adminEmail} eq ''))
   {
      $config->{adminEmail} = '';

      # guess admin id from hostname and user logged in
      my $hostname = `hostname`; 
      my $username = `whoami`;
      if ((defined $hostname) && ($hostname ne '') && 
          (defined $username) && ($username ne ''))
      {
         $hostname =~ s/[\r\n\t\s]//go;
         $username =~ s/[\n\r\t\s]//go;
         
         $config->{adminEmail} = $username.'@'.$hostname;
      }
   }

   if ((! defined $config->{database}) || ($config->{database} eq ''))
   { $config->{database} = ''; }
   if ((! defined $config->{dbusername}) || ($config->{dbusername} eq ''))
   { $config->{dbusername} = ''; }
   if ((! defined $config->{dbpassword}) || ($config->{dbpassword} eq ''))
   { $config->{dbpassword} = ''; }

   if ((! defined $config->{publisherauthority}) || ($config->{publisherauthority} eq ''))
   { $config->{publisherauthority} = 'University Libraries'; }
   if ((! defined $config->{publisherlocation}) || ($config->{publisherlocation} eq ''))
   { $config->{publisherlocation} = ''; }

   if ((! defined $config->{language}) || ($config->{language} eq ''))
   { $config->{language} = 'en'; }

   if ((! defined $config->{recordlimit}) || ($config->{recordlimit} eq ''))
   { $config->{recordlimit} = '100'; }
   if ((! defined $config->{resumptionseparator}) || ($config->{resumptionseparator} eq ''))
   { $config->{resumptionseparator} = '!'; }
   if ((! defined $config->{sampleIdentifier}) || ($config->{sampleIdentifier} eq ''))
   { $config->{sampleIdentifier} = 'etd-1234567890'; }

   if ((! defined $config->{content}) || ($config->{content} eq ''))
   { $config->{content} = 'Theses and Dissertations produced by students'; }
   if ((! defined $config->{metadataPolicy}) || ($config->{metadataPolicy} eq ''))
   { $config->{metadataPolicy} = 'Metadata may be used by commercial and non-commercial users'; }
   if ((! defined $config->{dataPolicy}) || ($config->{dataPolicy} eq ''))
   { $config->{dataPolicy} = 'Full texts are individually tagged and the rights statements must be adhered to'; }
   if ((! defined $config->{submissionPolicy}) || ($config->{submissionPolicy} eq ''))
   { $config->{submissionPolicy} = 'Submissions are only made internally by students'; }

   $config;   
}


# write configuration data to file cfg.pl
sub WriteConfig 
{
   my ($config) = @_;
   
   open (CONFIG, '>cfg.pl');
   print CONFIG "{\n";
   foreach my $key (keys %$config)
   {
      printf CONFIG "   %-20s => \'%s\',\n", $key, $config->{$key};
   }
   print CONFIG "}\n\n";
   close (CONFIG);
}


my $fastforward = 0;

# input a line of data, with prompt and default value
sub InputLine
{
   my ($variable, $prompt) = @_;
   
   if (defined $prompt)
   {
      print "$prompt [$$variable] : ";
   }
   if ($fastforward == 1)
   {
      print "$$variable\n";
      select(undef, undef, undef, 0.25);
   }
   else
   {
      my $line = <STDIN>;
      chomp $line;
      if ($line eq '&continue')
      {
         $fastforward = 1;
         print "$$variable\n";
      }
      elsif ($line eq '&delete')
      {
         $$variable = '';
         InputLine ($variable, $prompt);
      }
      elsif ($line ne '')
      {
         $$variable = $line;
      }
   }
}


# wait for user to press ENTER
sub InputEnter
{
   my $temp = '';
   InputLine (\$temp);
}


# check validity of input and print error if necessary
sub InvalidInput
{
   my ($check, $errormsg) = @_;
   
   if ($check)
   {
      print "$errormsg\n";
      $fastforward = 0;
      return 1;
   }
   0;
}


# check for database operations on a single table
sub CheckTable
{
   my ($dbh, $table) = @_;

   print "Checking $table ... ";
   my $sth = $dbh->prepare ("select * from $table");
   if (! defined $sth)
   {
      print "Error preparing a select for table $table\n";
      $dbh->disconnect; exit;
   }
   if (!($sth->execute))
   {
      print "Error executing a select on table $table\n";
      $sth->finish; $dbh->disconnect; exit;
   }
   if (!($sth->fetchrow_hashref))
   {
      print "Error getting row from table $table\n";
      $sth->finish; $dbh->disconnect; exit;
   }
   print "ok\n";
   $sth->finish;
}


# main program body !
sub main 
{
   $| = 1;
   my $config = ReadConfig;
   my $ok;
   
   print "+-----------------------------------------+\n".
         "| NDLTD OAI Data Provider Configurator    |\n".
         "+-----------------------------------------+\n".
         "| Version 1.0 :: September 2001           |\n".
         "| Hussein Suleman <hussein\@vt.edu>        |\n".
         "| Digital Library Research Laboratory     |\n".
         "| www.dlib.vt.edu :: Virginia Tech        |\n".
         "------------------------------------------+\n\n".
         "If you use the ETD-db software, this script will configure\n".
         "the OAI data provider scripts to work with your archive.\n".
         "The configuration is stored in cfg.pl but it is recommended\n".
         "that you use this program as it also does sanity checks.\n\n".
         "Defaults/previous values are in brackets - press <enter> to accept those\n".
         "enter \"&delete\" to erase a default value\n".
         "enter \"&continue\" to skip further questions and use all defaults\n".
         "press <ctrl>-c to escape at any time (new values will be lost)\n".
         "\nPress <enter> to continue\n\n";
         
   InputEnter;

         
   # test for not finding DBI
   my $gotdbi = eval "require DBI";
   print "\n[DBI]\n";
   print "You need the DBI Perl module installed in order to use this script\n".
         "Checking ... ";
   if (! defined $gotdbi)
   {
      print "not installed !\n".
            "Please press CTRL-C and then install that module.\n".
            "You also need a module specific to your database e.g., DBD::mysql.\n".
            " \"man CPAN\" for help on how to do this.\n\n";
      exit;
   }
   else
   {
      print "ok\n\n";
   }
   

   # test for available driver
   my @dbi_drivers = DBI->available_drivers;
   my $driver_list = join (', ', @dbi_drivers);
   print "\n[DATABASE DRIVER]\n";
   print "DBI requires special drivers to access different databases. Please\n".
         "confirm that the list below contains the name of your database.\n".
         "If it is not here or you are not sure, press CTRL-C and install it\n";
   print "\n".$driver_list."\n\n";
   print "Press ENTER to continue\n";
   InputEnter;

   
   # Database Name + Username + Password
   $ok = 0;
   print "\n[DATABASE CONNECTION]\n";
   print "The OAI scripts need to connect to your ETD database to obtain\n".
         "the metadata for your ETDs. For this you need to supply the name of the\n".
         "database as well as the hostname of the machine that runs it\n".
         "(you will be asked for username and password next).\n".
         "If you dont have permission to access the database from this host\n".
         "you may want to install this OAI script on another machine or grant\n".
         "permissions before you continue.\n";
   do {
      my @dbparts = split (/[:]/, $config->{database});
      if (! defined $dbparts[1]) { $dbparts[1] = 'mysql'; }
      if (! defined $dbparts[2]) { $dbparts[2] = 'etd_available'; }
      if (! defined $dbparts[3]) { $dbparts[3] = 'localhost'; }
   
      my $found = 0;
      do {
         print "\nDrivers: $driver_list\n";
         InputLine (\$dbparts[1], "Choose database driver from above list");
         foreach my $testdbidriver (@dbi_drivers)
         {
            if ($testdbidriver eq $dbparts[1])
            { $found = 1; }
         }
      } while (InvalidInput (($found == 0), "Invalid choice. Please choose again."));
      
      InputLine (\$dbparts[3], "Enter hostname/IP of database server");
      InputLine (\$dbparts[2], "Enter name of database");

      $dbparts[0] = 'DBI';
      $config->{database} = join (':', @dbparts);

      InputLine (\$config->{dbusername}, "Enter the username to be used to connect");
      InputLine (\$config->{dbpassword}, "Enter the password to be used to connect");

      print "Checking ... ";
      my $dbh = DBI->connect ($config->{database}, $config->{dbusername}, 
                              $config->{dbpassword}, { PrintError=>0 });
      if (! defined $dbh)
      { 
         print "Cannot connect to the specified database with user/pass. Try again.\n";
         $fastforward = 0;
      }
      else
      {
         print "ok\n";
         $dbh->disconnect;
         $ok = 1;
      }
   } while ($ok == 0);
   

   # Databases
   print "\n[DATABASE TABLES]\n";
   my $dbh = DBI->connect ($config->{database}, $config->{dbusername}, 
                           $config->{dbpassword}, { PrintError=>0 });
   CheckTable ($dbh, 'etd_main');
   CheckTable ($dbh, 'keyword_by_urn');
   CheckTable ($dbh, 'filename_by_urn');
   CheckTable ($dbh, 'advisor_by_urn');
   $dbh->disconnect;


   # Archive Id
   print "\n[ARCHIVE ID]\n";
   print "Each OAI-compliant archive ought to have a unique identifier associated\n".
         "with it. This is usually the common acronym for the institution.\n".
         "Examples: VT, MIT, UDW, HUBerlin.\n".
         "Restrictions: It has to be alphanumeric.\n\n";
   do {
      InputLine (\$config->{archiveId}, "Enter your Archive ID");
   } while (InvalidInput (($config->{archiveId} !~ /^[a-zA-Z0-9]+$/), 
                          "That seems to be non-alphanumeric. Please try again."));

   # set other defaults that are based on this
   my $iname = $config->{archiveId};
   $iname =~ s/^(.*)ETD$/$1/;
   if ((! defined $config->{repositoryName}) || ($config->{repositoryName} eq ''))
   { $config->{repositoryName} = $iname.' Electronic Thesis and Dissertation Archive'; }
   if ((! defined $config->{publisher}) || ($config->{publisher} eq ''))
   { $config->{publisher} = $iname; }


   # Repository Name
   print "\n[REPOSITORY NAME]\n";
   print "For identification purposes, you need to specify the full name of\n".
         "the repository\n".
         "Example: Virginia Tech Electronic Thesis and Dissertation Collection.\n\n";
   do {
      InputLine (\$config->{repositoryName}, "Enter your repository name");
   } while (InvalidInput (($config->{publisher} eq ''),
                          "This field cannot be left blank. Please enter a value."));
   

   # Admin Email
   print "\n[ADMINISTRATOR EMAIL]\n";
   print "When your repository is registered and used by others, any problems\n".
         "will be reported to an administator, whose email must be provided\n".
         "Examples: admin\@reposit.org, etds\@etdcentral.edu.\n\n";
   do {
      InputLine (\$config->{adminEmail}, "Enter your administrator's email");
   } while (InvalidInput (($config->{adminEmail} !~ /^[^@]+@[^@\.]+\.(.+)/), 
                          "That doesnt seem to be in the right format. Please try again."));


   # Publisher Information
   print "\n[PUBLISHER]\n";
   print "Most metadata formats require some information about the publisher\n".
         "of the ETD, which is usually the university that granted the degree.\n\n";
   do {
      InputLine (\$config->{publisher}, "Name of publisher/institution");
   } while (InvalidInput (($config->{publisher} eq ''),
                          "This field cannot be left blank. Please enter a value."));
   do {
      InputLine (\$config->{publisherauthority}, "Department responsible for ETDs");
   } while (InvalidInput (($config->{publisherauthority} eq ''),
                          "This field cannot be left blank. Please enter a value."));

   my $acronym = $config->{publisher};
   $acronym =~ s/[^A-Z]//go;
   if ((length ($acronym) > 1) && 
       ((! defined $config->{publisherinitials}) || ($config->{publisherinitials} eq '')))
   { $config->{publisherinitials} = $acronym; }
   do {
      InputLine (\$config->{publisherinitials}, "Acronym for publisher");
   } while (InvalidInput (($config->{publisherinitials} eq ''),
                          "This field cannot be left blank. Please enter a value."));
   do {
      InputLine (\$config->{publisherlocation}, "Location of publisher");
   } while (InvalidInput (($config->{publisherlocation} eq ''),
                          "This field cannot be left blank. Please enter a value."));


   # Language
   print "\n[LANGUAGE]\n";
   print "Example: en, en-uk, fr, es\n\n";
   do {
      InputLine (\$config->{language}, "Language for metadata");
   } while (InvalidInput (($config->{language} eq ''),
                          "This field cannot be left blank. Please enter a value."));


   # Sample Identifier
   print "\n[SAMPLE IDENTIFIER]\n";
   print "Example: etd-1234567890\n".
         "Note: Dont prepend an \"oai:archiveid:\"\n\n";
   do {
      InputLine (\$config->{sampleIdentifier}, "Sample identifier");
   } while (InvalidInput (($config->{sampleIdentifier} eq ''),
                          "This field cannot be left blank. Please enter a value."));


   # Record Limit
   print "\n[RECORD LIMIT]\n";
   print "This is the number of records disseminated in each batch.\n\n";
   do {
      InputLine (\$config->{recordlimit}, "No of records");
   } while (InvalidInput (($config->{recordlimit} < 1),
                          "This field must be numeric and >0. Please try again."));


   # Resumption Token Separator
   print "\n[RESUMPTION TOKEN SEPARATOR]\n";
   print "This character can be anything not found in an identifier, typically punctuation.\n".
         "Example: !, ?\n\n";
   do {
      InputLine (\$config->{resumptionseparator}, "Separator");
   } while (InvalidInput ((length ($config->{resumptionseparator}) != 1),
                          "The separator has to be exactly one character. Please try again."));


   # Policies
   print "\n[POLICIES]\n";
   print "As additional information, your archive should have policies for use of its data\n".
         "and metadata by others\n\n";
   do {
      InputLine (\$config->{content}, "What is the content of the archive");
   } while (InvalidInput (($config->{content} eq ''),
                          "This field cannot be left blank. Please enter a value."));
   do {
      InputLine (\$config->{dataPolicy}, "What is the usage policy for the ETDs (not the metadata)");
   } while (InvalidInput (($config->{dataPolicy} eq ''),
                          "This field cannot be left blank. Please enter a value."));
   do {
      InputLine (\$config->{metadataPolicy}, "What is the usage policy for the metadata");
   } while (InvalidInput (($config->{metadataPolicy} eq ''),
                          "This field cannot be left blank. Please enter a value."));
   do {
      InputLine (\$config->{submissionPolicy}, "What is the policy for making submissions");
   } while (InvalidInput (($config->{submissionPolicy} eq ''),
                          "This field cannot be left blank. Please enter a value."));


   WriteConfig ($config);
   
   print "\n\n\nFinis.\n\n";
   
   print "Now you can test the archive from the Repository Explorer at\n".
         "http://purl.org/net/oai_explorer - use the \"Test and Add\" option\n".
         "For a URL, use the web server path to the \"oai.pl\" script in this\n".
         "directory.\n".
         "After you confirm that it works properly, register your archive\n".
         "at http://www.openarchives.org and send email to \n".
         "etdunion\@oai.dlib.vt.edu so that we can add your collection to the\n".
         "NDLTD Union Catalog.\n\n";
}

main;
